"""Example usage of pystibmivb."""
import asyncio
import datetime
import json
import unittest

import aiohttp

from pystibmivb import STIBService, Passage, LineInfo, InvalidLineFilterException, NoScheduleFromAPIException
from tests.pystibmivb import MockAPIClient


class TestPassages(unittest.TestCase):
    def setUp(self):
        self.LOOP = asyncio.get_event_loop()


    def test_filtered_in_terminus(self):
        async def go(LOOP):
            stop_name = "scherdemael"
            lines_filter = [(46, "Glibert")]
            custom_session = aiohttp.ClientSession()

            APIClient = MockAPIClient()

            service = STIBService(APIClient)
            passages = await service.get_passages(stop_name, lines_filter, lang=('fr', 'fr'))

            now = datetime.datetime.now()
            delta1 = datetime.timedelta(minutes=3, seconds=25)
            delta2 = datetime.timedelta(minutes=13, seconds=22)

            # Check message
            self.assertEqual(passages[0]["message"], "foofr")
            self.assertEqual(passages[1]["message"], "")

        self.LOOP.run_until_complete(go(self.LOOP))


    def test_filtered_in_direction(self):
        async def go(LOOP):
            stop_name = "Scherdemael"
            lines_filter = [(46, 1)]
            custom_session = aiohttp.ClientSession()

            APIClient = MockAPIClient()

            service = STIBService(APIClient)
            passages = await service.get_passages(stop_name, lines_filter, lang=('fr', 'fr'))

            now = datetime.datetime.now()
            delta1 = datetime.timedelta(minutes=3, seconds=25)
            delta2 = datetime.timedelta(minutes=13, seconds=22)

            # Check message
            self.assertEqual(passages[0]["message"], "foofr")
            self.assertEqual(passages[1]["message"], "")

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_filtered_in_invalid_filter(self):
        async def go(LOOP):
            stop_name = "Scherdemael"
            lines_filter = [(104, 1)]
            custom_session = aiohttp.ClientSession()

            APIClient = MockAPIClient()

            service = STIBService(APIClient)

            hasRaised = False
            try:
                await service.get_passages(stop_name, lines_filter, lang=('fr', 'fr'))
            except InvalidLineFilterException:
                hasRaised = True

            self.assertTrue(hasRaised)

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_serializable(self):
        async def go(LOOP):
            stop_name = "scherdemael"
            lines_filter = [(46, "Glibert")]
            custom_session = aiohttp.ClientSession()

            APIClient = MockAPIClient()

            service = STIBService(APIClient)
            passages = await service.get_passages(stop_name, lines_filter)

            print(json.dumps(passages))

            await custom_session.close()

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_atomic_passage_serialization(self):
        now = datetime.datetime.now()
        delta1 = datetime.timedelta(minutes=3, seconds=25)
        p = Passage(stop_id=42, lineId=21, destination="FooDest",
                    expectedArrivalTime=(now + delta1).strftime("%Y-%m-%dT%H:%M:%S"),
                    lineInfos=LineInfo(line_nr=21, line_type="B", line_color="#FFFFFF", line_text_color="#000000"), message="FooMsg", now=now)

        js = json.loads(json.dumps(p))
        self.assertEqual(js["stop_id"], 42)
        self.assertEqual(js["line_id"], 21)
        self.assertEqual(js["destination"], "FooDest")
        self.assertEqual(js["expected_arrival_time"], (now + delta1).strftime("%Y-%m-%dT%H:%M:%S"))
        self.assertEqual(js["line_color"], "#FFFFFF")
        self.assertEqual(js["line_text_color"], "#000000")
        self.assertEqual(js["line_type"], "B")
        self.assertEqual(js["message"], "FooMsg")
        self.assertEqual(js["arriving_in"]["min"], 3)
        self.assertEqual(js["arriving_in"]["sec"], 24)

    def test_empty_response_crawls_for_data(self):
        async def go(LOOP):
            stop_name = "De Brouckère"
            lines_filter = [(5, 1)]
            custom_session = aiohttp.ClientSession()

            APIClient = MockAPIClient()

            service = STIBService(APIClient)
            exception = "Unraised"
            try:
                passages = await service.get_passages(stop_name, lines_filter, now=datetime.datetime(2020, 1, 28, hour=23, minute=59, second=52))
            except NoScheduleFromAPIException as e:
                exception = "Raised"
                passages = e.get_next_passages()
            self.assertEqual(exception, "Raised")
            self.assertGreaterEqual(len(passages), 1)
            self.assertEqual(passages[0]['expected_arrival_time'], '2020-01-29T00:06:31')


        self.LOOP.run_until_complete(go(self.LOOP))


if __name__ == '__main__':
    unittest.main()



"""Common attributes and functions."""
import datetime
import json

from pystibmivb import AbstractSTIBAPIClient
from pystibmivb.client.STIBAPIClient import PASSING_TIME_BY_POINT_SUFFIX
from pystibmivb.service.ShapefileService import ENDPOINT_SHAPEFILES, ENDPOINT_GTFS


class MockAPIClient(AbstractSTIBAPIClient):
    """A class for common functions."""

    async def _api_call_passingTimeByPoint_for_10_stop_ids(self, point_ids: list) -> dict:
        return await self.api_call(PASSING_TIME_BY_POINT_SUFFIX + "%2C".join(point_ids))

    async def api_call_passingTimeByPoint_for_stop_ids(self, point_ids: set) -> dict:
        res = {"points": []}
        subset_point_ids = []
        for i in range(1, len(point_ids)+1):
            subset_point_ids.append(point_ids.pop())
            if i % 10 == 0 or len(point_ids) == 0:
                subset_res = await self._api_call_passingTimeByPoint_for_10_stop_ids(subset_point_ids)
                res["points"].extend(subset_res["points"])
                subset_point_ids = []
        return res

    async def api_call_passingTimeByPoint_for_stop_id(self, point_id: str):
        if point_id == "3755":
            now = datetime.datetime.now()
            delta1 = datetime.timedelta(minutes=3, seconds=25)
            delta2 = datetime.timedelta(minutes=13, seconds=22)
            return json.loads('''{"points": [
                            {"passingTimes": [
                                {
                                 "destination": 
                                    {"fr": "WTC / GLIBERT", "nl": "WTC / GLIBERT"}, 
                                 "expectedArrivalTime": "''' + (now + delta1).strftime("%Y-%m-%dT%H:%M:%S") + '''+01:00", 
                                 "message": 
                                    {"fr": "foofr", "nl": "foonl"},
                                 "lineId": "46"
                                }, 
                                {
                                 "destination": 
                                    {"fr": "WTC / GLIBERT", "nl": "WTC / GLIBERT"}, 
                                 "expectedArrivalTime": "''' + (now + delta2).strftime("%Y-%m-%dT%H:%M:%S") + '''+01:00", 
                                 "lineId": "46"
                                }
                            ], 
                             "pointId": "3755"
                            }
                        ]
                    }''')
        if point_id == "8012":
            now = datetime.datetime.now()
            delta1 = datetime.timedelta(minutes=3, seconds=25)
            delta2 = datetime.timedelta(minutes=13, seconds=22)
            return json.loads('''{"points": [{"passingTimes": [], "pointId": "8012"}]}''')

    def __init__(self):
        with open("../resources/shapefiles.zip", 'rb') as sf:
            self.shapefilezipcontent = sf.read()
        with open("../resources/gtfs.zip", 'rb') as sf:
            self.gtfszipcontent = sf.read()

    async def api_call(self, endpoint_suffix: str, additional_headers=None):
        if endpoint_suffix == ENDPOINT_SHAPEFILES:
            return self.shapefilezipcontent
        elif endpoint_suffix == ENDPOINT_GTFS:
            return self.gtfszipcontent
        elif endpoint_suffix.startswith(PASSING_TIME_BY_POINT_SUFFIX):
            if endpoint_suffix.endswith("3755"):
                now = datetime.datetime.now()
                delta1 = datetime.timedelta(minutes=3, seconds=25)
                delta2 = datetime.timedelta(minutes=13, seconds=22)
                return json.loads('''{"points": [
                                {"passingTimes": [
                                    {
                                     "destination": 
                                        {"fr": "WTC / GLIBERT", "nl": "WTC / GLIBERT"}, 
                                     "expectedArrivalTime": "'''+(now+delta1).strftime("%Y-%m-%dT%H:%M:%S")+'''+01:00", 
                                     "message": 
                                        {"fr": "foofr", "nl": "foonl"},
                                     "lineId": "46"
                                    }, 
                                    {
                                     "destination": 
                                        {"fr": "WTC / GLIBERT", "nl": "WTC / GLIBERT"}, 
                                     "expectedArrivalTime": "'''+(now+delta2).strftime("%Y-%m-%dT%H:%M:%S")+'''+01:00", 
                                     "lineId": "46"
                                    }
                                ], 
                                 "pointId": "3755"
                                }
                            ]
                        }''')
            if endpoint_suffix.endswith("8012"):
                now = datetime.datetime.now()
                delta1 = datetime.timedelta(minutes=3, seconds=25)
                delta2 = datetime.timedelta(minutes=13, seconds=22)
                return json.loads('''{"points": [{"passingTimes": [], "pointId": "8012"}]}''')



import asyncio
import unittest
from datetime import datetime

from pystibmivb import ShapefileService
from tests.pystibmivb import MockAPIClient


class TestShapefileReader(unittest.TestCase):
    def setUp(self):
        self.LOOP = asyncio.get_event_loop()

    def test_filtered_out(self):
        async def check_color(sf_reader, line_id, expected_text_color):
            l = await sf_reader.get_line_info(line_id)
            self.assertEqual(l.get_line_text_color(), expected_text_color)

        async def go(LOOP):
            APIClient = MockAPIClient()

            sf_reader = ShapefileService(APIClient)

            frinfo = await sf_reader.get_stop_infos("Scherdemael")
            nlinfo = await sf_reader.get_stop_infos("Scherdemaal")

            self.assertEqual(frinfo.get_line_info(46).get_line_color(), nlinfo.get_line_info(46).get_line_color())
            self.assertEqual(frinfo.get_line_info(46).get_line_type(), nlinfo.get_line_info(46).get_line_type())
            self.assertEqual(frinfo.get_line_info(46).get_line_nr(), nlinfo.get_line_info(46).get_line_nr())

            l46 = await sf_reader.get_line_info(46)
            self.assertEqual(l46.get_line_color(), "#DE3B21")
            self.assertEqual(l46.get_line_text_color(), "#FFFFFF")
            self.assertEqual(l46.get_line_type(), "B")
            self.assertEqual(l46.get_line_nr(), 46)

            await check_color(sf_reader, 3, "#000000")
            await check_color(sf_reader, 5, "#FFFFFF")
            await check_color(sf_reader, 6, "#FFFFFF")

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_location(self):
        # https://xkcd.com/2170/
        async def go(LOOP):
            APIClient = MockAPIClient()

            sf_reader = ShapefileService(APIClient)

            info = await sf_reader.get_stop_infos("Scherdemael")

            self.assertAlmostEqual(info.get_location()["lat"], 50.8311, delta=0.0001)
            self.assertAlmostEqual(info.get_location()["lon"], 4.2896, delta=0.0001)

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_locations(self):
        # https://xkcd.com/2170/
        async def go(LOOP):
            APIClient = MockAPIClient()

            sf_reader = ShapefileService(APIClient)

            info = await sf_reader.get_stop_infos("Scherdemael")

            print(info.get_locations())
            self.assertAlmostEqual(info.get_locations()['3755']["lat"], 50.8312, delta=0.0001)
            self.assertAlmostEqual(info.get_locations()['3755']["lon"], 4.2900, delta=0.0001)

            self.assertAlmostEqual(info.get_locations()['3713']["lat"], 50.8309, delta=0.0001)
            self.assertAlmostEqual(info.get_locations()['3713']["lon"], 4.2892, delta=0.0001)

        self.LOOP.run_until_complete(go(self.LOOP))

    def test_gtfs_next_passage(self):
        # https://xkcd.com/2170/
        async def go(LOOP):
            APIClient = MockAPIClient()

            sf_reader = ShapefileService(APIClient)

            now = datetime.now()
            info = await sf_reader.get_next_stop_passage('3755', now)
            self.assertLess(now, info)
            info = await sf_reader.get_next_stop_passage('3755', datetime(2020, 1, 28, hour=2, minute=32, second=2))
            self.assertEqual(info, datetime(2020, 1, 28, hour=5, minute=7, second=27))
            info = await sf_reader.get_next_stop_passage('3755', datetime(2020, 1, 28, hour=5, minute=10, second=2))
            self.assertEqual(info, datetime(2020, 1, 28, hour=5, minute=18, second=27))
            info = await sf_reader.get_next_stop_passage('3755', datetime(2020, 1, 28, hour=23, minute=58, second=2))
            self.assertEqual(info, datetime(2020, 1, 28, hour=23, minute=59, second=51))
            info = await sf_reader.get_next_stop_passage('3755', datetime(2020, 1, 28, hour=23, minute=59, second=52))
            self.assertEqual(info, datetime(2020, 1, 29, hour=5, minute=7, second=27))

        self.LOOP.run_until_complete(go(self.LOOP))


if __name__ == '__main__':
    unittest.main()



"""Example usage of pystibmivb."""
import asyncio
import unittest

from pystibmivb import STIBService
from tests.pystibmivb import MockAPIClient


class TestPassages(unittest.TestCase):
    def setUp(self):
        self.LOOP = asyncio.get_event_loop()

    def tearDown(self):
        self.LOOP.close()

    def test_filtered_out(self):
        async def go(LOOP):
            stop_name = "scherdemael"
            lines_filter = [(46, "Glibert")]

            APIClient = MockAPIClient()

            service = STIBService(APIClient)
            print(await service.get_passages(stop_name, lines_filter))

        self.LOOP.run_until_complete(go(self.LOOP))


if __name__ == '__main__':
    unittest.main()






import unittest
from pystibmivb import STIBAPIClient, ShapefileService
import aiohttp
import asyncio

CLIENT_ID = 'Wirff1HT1tTH7mLX1dMQAbOEHDoa' # Put your openapi client ID here
CLIENT_SECRET = 'tYKqSKbmjw3hKsoNtaaKKtXXP0sa' # Put your openapi client secret here

class TestIntegrationApiCalls(unittest.TestCase):
    def setUp(self):
        self.LOOP = asyncio.get_event_loop()


    def test_filtered_in_terminus(self):
        async def go(LOOP):
            custom_session = aiohttp.ClientSession()

            APIClient = STIBAPIClient(LOOP, custom_session, CLIENT_ID, CLIENT_SECRET)
            shapefileService = ShapefileService(APIClient)


            print(await APIClient.api_call_passingTimeByPoint_for_stop_ids((await shapefileService.get_stop_infos("Scherdemael")).get_stop_ids()))

        self.LOOP.run_until_complete(go(self.LOOP))


if __name__ == '__main__':
    unittest.main()
